from rdflib import Graph, URIRef, Literal
from urllib.parse import quote

# 加载之前生成的RDF图
g = Graph()
g.parse('university_knowledge_graph.ttl', format='turtle')

# 定义SPARQL查询模板
SPARQL_TEMPLATES = {
    "类型": """
        PREFIX cns: <http://cnschema.org/>
        PREFIX ex: <http://example.org/>
        SELECT ?typeName WHERE {
            ex:%s cns:hasClassification ?type .
            ?type cns:name ?typeName .
        }
    """,
    "英文名": """
        PREFIX cns: <http://cnschema.org/>
        PREFIX ex: <http://example.org/>
        SELECT ?name WHERE {
            ex:%s cns:alternateName ?englishName .
            ?englishName cns:name ?name .
        }
    """,
    "简称": """
        PREFIX cns: <http://cnschema.org/>
        PREFIX ex: <http://example.org/>
        SELECT ?name WHERE {
            ex:%s cns:abbreviation ?abbr .
            ?abbr cns:name ?name .
        }
    """,
    "创办时间": """
        PREFIX cns: <http://cnschema.org/>
        PREFIX ex: <http://example.org/>
        SELECT ?dateValue WHERE {
            ex:%s cns:foundingDate ?date .
            ?date cns:date ?dateValue .
        }
    """,
    "地址": """
        PREFIX cns: <http://cnschema.org/>
        PREFIX ex: <http://example.org/>
        SELECT ?name WHERE {
            ex:%s cns:location ?address .
            ?address cns:name ?name .
        }
    """,
    "院系设置": """
        PREFIX cns: <http://cnschema.org/>
        PREFIX ex: <http://example.org/>
        SELECT ?name WHERE {
            ex:%s cns:hasOrganizationUnit ?depart .
            ?depart cns:name ?name .
        }
    """
}

# 编码URI中的特殊字符
def encode_uri(name):
    return quote(name, safe='')

# 执行SPARQL查询
def execute_sparql_query(query_template, entity_name):
    query = query_template % encode_uri(entity_name)
    results = g.query(query)
    return results

# 处理用户输入的问题
def answer_question(question):
    # 简单的问题解析
    if "类型" in question:
        entity_name = question.replace("的类型是什么", "").strip()
        query_template = SPARQL_TEMPLATES["类型"]
    elif "英文名" in question:
        entity_name = question.replace("的英文名是什么", "").strip()
        query_template = SPARQL_TEMPLATES["英文名"]
    elif "简称" in question:
        entity_name = question.replace("的简称是什么", "").strip()
        query_template = SPARQL_TEMPLATES["简称"]
    elif "创办时间" in question:
        entity_name = question.replace("的创办时间是什么", "").strip()
        query_template = SPARQL_TEMPLATES["创办时间"]
    elif "地址" in question:
        entity_name = question.replace("的地址是什么", "").strip()
        query_template = SPARQL_TEMPLATES["地址"]
    elif "院系设置" in question:
        entity_name = question.replace("的院系设置是什么", "").strip()
        query_template = SPARQL_TEMPLATES["院系设置"]
    else:
        return "抱歉，我无法理解这个问题。"

    # 执行查询并返回结果
    results = execute_sparql_query(query_template, entity_name)
    if results:
        return "\n".join([str(row[0]) for row in results])
    else:
        return "没有找到相关信息。"

# 主函数
def main():
    while True:
        question = input("请输入您的问题（或输入'退出'以结束）：")
        if question == "退出":
            break
        answer = answer_question(question)
        print(answer)

if __name__ == "__main__":
    main()