import json
from rdflib import Graph, Namespace, Literal, URIRef
from urllib.parse import quote  # 导入 quote 函数用于编码 URI

# 定义命名空间
CNS = Namespace("http://cnschema.org/")
EX = Namespace("http://example.org/")

# 初始化 RDF 图
g = Graph()

# 绑定命名空间
g.bind("cns", CNS)
g.bind("ex", EX)

# 编码 URI 中的特殊字符
def encode_uri(name):
    return quote(name, safe='')  # 对特殊字符进行编码

# 读取数据并构建 RDF 图
def build_rdf_graph():
    # 读取 all.json 文件
    with open('all.json', 'r', encoding='utf-8') as fr:
        full_data = json.load(fr)

    # 添加中央节点 "大学"
    university_uri = EX[encode_uri("大学")]
    g.add((university_uri, CNS.type, CNS.Organization))
    g.add((university_uri, CNS.name, Literal("大学", lang="zh")))

    # 处理类型节点
    with open('Type.txt', 'r', encoding='utf-8') as fr:
        for line in fr.readlines():
            tmp = line.strip('\n')
            for key, value in eval(tmp).items():
                type_uri = EX[encode_uri(value)]
                g.add((type_uri, CNS.type, CNS.Classification))
                g.add((type_uri, CNS.name, Literal(value, lang="zh")))
                g.add((university_uri, CNS.hasClassification, type_uri))

    # 处理英文名节点
    with open('English.txt', 'r', encoding='utf-8') as fr:
        for line in fr.readlines():
            tmp = line.strip('\n')
            for key, value in eval(tmp).items():
                english_uri = EX[encode_uri(value)]
                g.add((english_uri, CNS.type, CNS.Name))
                g.add((english_uri, CNS.name, Literal(value, lang="en")))

    # 处理简称节点
    with open('Abbr.txt', 'r', encoding='utf-8') as fr:
        for line in fr.readlines():
            tmp = line.strip('\n')
            for key, value in eval(tmp).items():
                abbr_uri = EX[encode_uri(value)]
                g.add((abbr_uri, CNS.type, CNS.Abbreviation))
                g.add((abbr_uri, CNS.name, Literal(value, lang="zh")))

    # 处理时间节点
    with open('Time.txt', 'r', encoding='utf-8') as fr:
        for line in fr.readlines():
            tmp = line.strip('\n')
            for key, value in eval(tmp).items():
                time_uri = EX[encode_uri(value)]
                g.add((time_uri, CNS.type, CNS.Date))
                g.add((time_uri, CNS.date, Literal(value)))

    # 处理地址节点
    with open('Addr.txt', 'r', encoding='utf-8') as fr:
        for line in fr.readlines():
            tmp = line.strip('\n')
            for key, value in eval(tmp).items():
                addr_uri = EX[encode_uri(value)]
                g.add((addr_uri, CNS.type, CNS.Place))
                g.add((addr_uri, CNS.name, Literal(value, lang="zh")))

    # 处理院系节点
    with open('Depart.txt', 'r', encoding='utf-8') as fr:
        for line in fr.readlines():
            tmp = line.strip('\n')
            for key, value in eval(tmp).items():
                depart_uri = EX[encode_uri(value)]
                g.add((depart_uri, CNS.type, CNS.OrganizationUnit))
                g.add((depart_uri, CNS.name, Literal(value, lang="zh")))

    # 处理 all.json 中的数据
    for item in full_data:
        # 添加中文名节点
        name_uri = EX[encode_uri(item['中文名'])]
        g.add((name_uri, CNS.type, CNS.Organization))
        g.add((name_uri, CNS.name, Literal(item['中文名'], lang="zh")))

        # 链接类型节点
        type_uri = EX[encode_uri(item['类型'])]
        g.add((name_uri, CNS.hasClassification, type_uri))

        # 链接英文名节点
        english_uri = EX[encode_uri(item['英文名'])]
        g.add((name_uri, CNS.alternateName, english_uri))

        # 链接简称节点
        abbr_uri = EX[encode_uri(item['简称'])]
        g.add((name_uri, CNS.abbreviation, abbr_uri))

        # 链接时间节点
        time_uri = EX[encode_uri(item['创办时间'])]
        g.add((name_uri, CNS.foundingDate, time_uri))

        # 链接地址节点
        addr_uri = EX[encode_uri(item['地址'])]
        g.add((name_uri, CNS.location, addr_uri))

        # 链接院系节点
        depart_uri = EX[encode_uri(item['院系设置'])]
        g.add((name_uri, CNS.hasOrganizationUnit, depart_uri))

# 构建 RDF 图
build_rdf_graph()

# 将 RDF 图保存为 Turtle 格式
g.serialize(destination='university_knowledge_graph.ttl', format='turtle')
print("RDF 图已保存为 university_knowledge_graph.ttl")